from stats_collector.stats import Statistics
from stats_collector import FRED_DB
from stats_collector.stats.stat_registry import stat_registry
from ..groups import MOJEID_DAILY_GROUP


@stat_registry.register(MOJEID_DAILY_GROUP)
class MojeidContactsByVerificationLevel(Statistics):
    sql = '''
         SELECT
            highest_verification_level.handle, count(*)
          FROM
            (
                SELECT
                    CASE WHEN mojeid_verification_states_grouped.handle_arr = array_sort_dist('{conditionallyIdentifiedContact}'::varchar[])
                            THEN 'conditionallyIdentifiedContact'
                         WHEN mojeid_verification_states_grouped.handle_arr = array_sort_dist('{conditionallyIdentifiedContact,identifiedContact}'::varchar[])
                            THEN 'identifiedContact'
                         WHEN mojeid_verification_states_grouped.handle_arr = array_sort_dist('{conditionallyIdentifiedContact,identifiedContact,validatedContact}'::varchar[])
                            THEN 'validatedContact'
                         WHEN mojeid_verification_states_grouped.handle_arr = array_sort_dist('{conditionallyIdentifiedContact,validatedContact}'::varchar[])
                            THEN 'validatedContact'
                    END AS handle
                  FROM
                    (
                        SELECT
                            array_sort_dist(array_agg(eos.name)) AS handle_arr, os.object_id
                          FROM
                            object_state os
                            JOIN enum_object_states eos ON eos.id = os.state_id
                          WHERE
                            os.object_id IN
                                (
                                    SELECT
                                        os.object_id
                                      FROM
                                        object_state os
                                        JOIN enum_object_states eos ON eos.id = os.state_id
                                      WHERE
                                        eos.name = 'mojeidContact'
                                        AND os.valid_from <= %(midnight)s
                                        AND (os.valid_to IS NULL OR os.valid_to > %(midnight)s)
                                )
                            AND eos.name IN ('conditionallyIdentifiedContact', 'identifiedContact', 'validatedContact')
                            AND os.valid_from <= %(midnight)s
                            AND (os.valid_to IS NULL OR os.valid_to > %(midnight)s)
                          GROUP BY 2
                    ) AS mojeid_verification_states_grouped
            ) AS highest_verification_level
            WHERE
                highest_verification_level.handle IN ('conditionallyIdentifiedContact', 'identifiedContact', 'validatedContact')
          GROUP BY 1;
    '''
    name = 'mojeid_contacts_by_verification_level'
    stat_db = FRED_DB

